/* ----------------------------------------------------------------------------- 
 * htoi.cxx
 *
 *     Produce a .i file given input.
 * 
 * Author(s) : David Beazley (beazley@cs.uchicago.edu)
 *
 * XML support (swigxml):
 *     Dave Kuhlman
 *     dkuhlman@rexx.com
 *     http://www.rexx.com/~dkuhlman
 *
 * Copyright (C) 1998-2000.  The University of Chicago
 * Copyright (C) 1995-1998.  The University of Utah and The Regents of the
 *                           University of California.
 *
 * See the file LICENSE for information on usage and redistribution.	
 * ----------------------------------------------------------------------------- */

static char cvsroot[] = "$Header: /cvsroot/SWIG/Source/Modules1.1/Attic/htoi.cxx,v 1.1.2.1 2001/10/14 17:25:12 beazley Exp $";

#include "mod11.h"
#include "preprocessor.h"
#include "xml.h"

// ----------------------------------------------

// Private globals:

static int Level;
static String * OutFileName = 0;
static FILE * OutFile = 0;
static int wrappingVariable = 0;

// ----------------------------------------------

int Printflvl(int level, DOH * outfile, const char *format, ...) {
	va_list ap;
	int ret;

	int idx;
	for (idx = 0; idx < level; idx++)
	{
		Printf(outfile, "    ");
	} // for

	va_start(ap,format);
	ret = vPrintf(outfile, format, ap);
	va_end(ap);
	return ret;
}

static char *usage = (char *)"\
XML Options (available with -xml)\n\
     -outfile file    - Write output to file (else stdout)\n\n"
     ;

/* -----------------------------------------------------------------------------
 * XML::main()
 * ----------------------------------------------------------------------------- */
void
XML::main(int argc, char *argv[]) {

	for (int i = 1; i < argc; i++) {
		if (argv[i])
		{
			if (strcmp(argv[i],"-help") == 0)
			{
				fputs(usage,stderr);
			}
			else if (strcmp (argv[i], "-outfile") == 0)
			{
				if (argv[i+1])
				{
					OutFileName = NewString(argv[i+1]);
					Swig_mark_arg(i);
					Swig_mark_arg(i+1);
					i++;
				}
				else
				{
					Swig_arg_error();
				} // if
			} // if
		} // if
	} // for

	if (OutFileName)
	{
		if ((OutFile = fopen(Char(OutFileName), "w")) == NULL)
		{
			SWIG_exit(EXIT_FAILURE);
		} // if
	}
	else
	{
		OutFile = stdout;
	} // if

} // main

/* ----------------------------------------------------------------------
 * XML::top()   - Top of parsing tree 
 * ---------------------------------------------------------------------- */

int XML::top(Node * node)
{
	int ret;
	Level = 0;
	Printflvl(Level, OutFile, "<?xml version=\"1.0\"?>\n");
	Printflvl(Level, OutFile, "<swig:definitions\n");
	Printflvl(Level, OutFile,
		"    xmlns:swig=\"http://www.rexx.com/~dkuhlman/swig_ns.xml\"\n");
	Printflvl(Level, OutFile, ">\n");
	Level++;
	ret = emit_children(node);
	Level--;
	Printflvl(Level, OutFile, "</swig:definitions>\n");
} // top

/* ----------------------------------------------------------------------
 * XML::classHandler()
 * ---------------------------------------------------------------------- */

int XML::classHandler(Node * node) 
{
	String * name = Getattr(node,"sym:name");
	
	/* Emit all of the class members 
	*/
	Printflvl(Level, OutFile, "<swig:classWrapper>\n");
	Level++;
	Printflvl(Level, OutFile, "<swig:className>%s</swig:className>\n",
		name);
	emit_children(node);
	Level--;
	Printf(OutFile, "    </swig:classWrapper>\n");

	return SWIG_OK;
} // classHandler

/* ----------------------------------------------------------------------
 * XML::functionWrapper()
 * ---------------------------------------------------------------------- */

int XML::functionWrapper(Node * node) {
	// String   *name   = Getattr(node,"sym:name");  // prefixed with class name
	if (wrappingVariable)
	{
		variableWrapper(node);
		return SWIG_OK;
	} // if

	String   * name   = Getattr(node,"name");
	String   * symname   = Getattr(node,"sym:name");
	SwigType * type   = Getattr(node,"type");
	ParmList * parms  = Getattr(node,"parms");
	
	Parm * parm;
	int idx;
	char buf[1024];

	Printflvl(Level, OutFile, "<swig:functionWrapper>\n");
	Level++;
	Printflvl(Level, OutFile, "<swig:functionName>%s</swig:functionName>\n",
		name);
	Printflvl(Level, OutFile, "<swig:functionSymName>%s</swig:functionSymName>\n",
		symname);
	Printflvl(Level, OutFile, "<swig:returnType>%s</swig:returnType>\n",
		SwigType_str(type, 0));
	Printflvl(Level, OutFile, "<swig:swigReturnType>%s</swig:swigReturnType>\n",
		Char(type));

	Printflvl(Level, OutFile, "<swig:parameters>\n");
	int num_arguments = emit_num_arguments(parms);
	Level++;
	Printflvl(Level, OutFile,
		"<swig:parameterCount>%d</swig:parameterCount>\n",
		num_arguments);
	for (idx = 0, parm = parms; idx < num_arguments; idx++)
	{

		while (Getattr(parm,"tmap:ignore"))
		{
			parm = Getattr(parm,"tmap:ignore:next");
		} // while

		SwigType *ptype = Getattr(parm, "type");
		String   *pname = Getattr(parm, "name");
		String   *lname = Getattr(parm, "lname");

		Printflvl(Level, OutFile, "<swig:parameter>\n");
		Level++;
		Printflvl(Level, OutFile,
			"<swig:parameterName>%s</swig:parameterName>\n",
			pname);
		Printflvl(Level, OutFile,
			"<swig:parameterType>%s</swig:parameterType>\n",
			SwigType_str(ptype, 0));
		Level--;
		Printflvl(Level, OutFile, "</swig:parameter>\n");
		parm = nextSibling(parm);
	} // for

	Level--;
	Printflvl(Level, OutFile, "</swig:parameters>\n");


//	Printflvl(Level, OutFile,"           action : %s\n", Getattr(n,"wrap:action")); 

	Level--;
	Printflvl(Level, OutFile,"</swig:functionWrapper>\n");
	return SWIG_OK;
} // functionWrapper


/* ----------------------------------------------------------------------
 * XML::membervariableHandler()
 * ---------------------------------------------------------------------- */

int XML::membervariableHandler(Node * node)
{
	wrappingVariable = 1;
	Language::membervariableHandler(node);
	wrappingVariable = 0;
	return SWIG_OK;
} //  membervariableHandler

/* ----------------------------------------------------------------------
 * XML::variableWrapper()
 * ---------------------------------------------------------------------- */

int XML::variableWrapper(Node * node)
{
	String   * name   = Getattr(node, "sym:name");
	String   * symname   = Getattr(node,"sym:name");
	SwigType * type   = Getattr(node,"type");

	Parm * parm;
	int idx;
	char buf[1024];

	Printflvl(Level, OutFile, "<swig:variableWrapper>\n");
	Level++;
	Printflvl(Level, OutFile, "<swig:variableName>%s</swig:variableName>\n",
		name);
	Printflvl(Level, OutFile, "<swig:variableSymName>%s</swig:variableSymName>\n",
		name);
	Printflvl(Level, OutFile, "<swig:type>%s</swig:type>\n",
		SwigType_str(type, 0));
	Printflvl(Level, OutFile, "<swig:swigType>%s</swig:swigType>\n",
		Char(type));

	Level--;
	Printflvl(Level, OutFile, "</swig:variableWrapper>\n");
	return SWIG_OK;
} // variableWrapper
